/// ////////////////////////////////////// ///
///                                        ///
///      DynRPG EventScript Debugger       ///
///      Written by Kazesui                ///
///      Version 1.00                      ///
///                                        ///
/// ////////////////////////////////////// ///

#define AUTO_DLLMAIN
#include <DynRPG/DynRPG.h>
#include <sstream>
#include "main.h"

std::map<int,bool>switchWatch;
std::map<int,int>variableWatch;
bool isScriptDebugging = false;

bool onComment( const char* text,
                const RPG::ParsedCommentData* 	parsedData,
                RPG::EventScriptLine* 	nextScriptLine,
                RPG::EventScriptData* 	scriptData,
                int 	eventId,
                int 	pageId,
                int 	lineId,
                int* 	nextLineId )
{
    std::string cmd = parsedData->command;
    if(!cmd.compare("assert_switch")){
        assert_switch(parsedData->parameters);
        return false;
    }
    if(!cmd.compare("assert_variable")){
        assert_variable(parsedData->parameters);
        return false;
    }
    if(!cmd.compare("add_switch_watch")){
        int id = parsedData->parameters[0].number;
        switchWatch[id] = RPG::switches[id];
        return false;
    }
    if(!cmd.compare("add_variable_watch")){
        int id = parsedData->parameters[0].number;
        variableWatch[id] = RPG::variables[id];
        return false;
    }
    if(!cmd.compare("clear_switch_watch")){
        switchWatch.clear();
        return false;
    }
    if(!cmd.compare("clear_variable_watch")){
        variableWatch.clear();
        return false;
    }
    if(!cmd.compare("activate_script_debugging")){
        isScriptDebugging = true;
        return false;
    }
    if(!cmd.compare("deactivate_script_debugging")){
        isScriptDebugging = false;
        return false;
    }
    return true;
}

void assert_switch(const RPG::ParsedCommentParameter* arg)
{
    bool value = (arg[1].text[1] == 'N' || arg[1].text[1] == 'n')?true:false;
    if(RPG::switches[arg[0].number] != value)
        MessageBox(NULL, arg[2].text , "Switch Assertion Failure" , MB_ICONERROR);
}

void assert_variable(const RPG::ParsedCommentParameter* arg)
{
    if(RPG::variables[arg[0].number] != arg[1].number)
        MessageBox(NULL, arg[2].text , "Variable Assertion Failure" , MB_ICONERROR);
}

bool onEventCommand( RPG::EventScriptLine *scriptLine,
                     RPG::EventScriptData *scriptData,
                     int eventId,
                     int pageId,
                     int lineId,
                     int *nextLineId )
{
    std::stringstream sw;
    if( switchWatch.size() > 0){
        bool update = false;
        if(eventId > 0){
            sw << "EventID: " << eventId << std::endl;
            sw << "EventPage: " << pageId << std::endl;
        } else {
            sw << "Common EventID: " << -eventId << std::endl;
        }
        sw << "EventLine: " << lineId << std::endl << std::endl;
        std::map<int,bool>::iterator itr = switchWatch.begin();
        while(itr != switchWatch.end()){
            if(RPG::switches[itr->first] != itr->second){
                update = true;
                sw << "Switch[" << itr->first <<"] changed from " << itr->second;
                sw << " to " << RPG::switches[itr->first] << std::endl;
                itr->second = RPG::switches[itr->first];
            }
            itr++;
        }
        if(update) MessageBox(NULL,sw.str().c_str(), "Switch Watchlist Update" , MB_ICONINFORMATION);
    }
    std::stringstream vw;
    if( variableWatch.size() > 0){
        bool update = false;
        if(eventId > 0){
            vw << "EventID: " << eventId << std::endl;
            vw << "EventPage: " << pageId << std::endl;
        } else {
            vw << "Common EventID: " << -eventId << std::endl;
        }
        vw << "EventLine: " << lineId << std::endl << std::endl;
        std::map<int,int>::iterator itr = variableWatch.begin();
        while(itr != variableWatch.end()){
            if(RPG::variables[itr->first] != itr->second){
                update = true;
                vw << "Switch[" << itr->first <<"] changed from " << itr->second;
                vw << " to " << RPG::variables[itr->first] << std::endl;
                itr->second = RPG::variables[itr->first];
            }
            itr++;
        }
        if(update) MessageBox(NULL,vw.str().c_str(), "Switch Watchlist Update" , MB_ICONINFORMATION);
    }
    std::stringstream ss;
    if( isScriptDebugging ){
        if(eventId > 0){
            ss << "EventID: " << eventId << std::endl;
            ss << "EventName: " << RPG::map->events[eventId]->eventData->name << std::endl;
            ss << "EventPage: " << pageId << std::endl;
        } else {
            ss << "Common EventID: " << -eventId << std::endl;
        }
        ss << "EventLine: " << lineId + 1 << std::endl << std::endl;
        ss << "Command: ";
        switch( scriptLine->command ){
            case RPG::EVCMD_CHANGE_SWITCH:
                ss << "Switch Operations" << std::endl;
                if( !scriptLine->parameter(0) ){
                    ss << "Operation: Set Switch[" << scriptLine->parameter(1) << "] ";
                    ss << getSwitchOp( scriptLine->parameter(3) ) << std::endl << std::endl;
                    ss << "Switch[" << scriptLine->parameter(1) << "] before operation = ";
                    if(RPG::switches[scriptLine->parameter(1)])ss<<"ON";else ss<<"OFF";
                } else if ( scriptLine->parameter(0) == 1 ){
                    ss << "Operation: Set Switch[ " << scriptLine->parameter(1);
                    ss << " - " << scriptLine->parameter(2) << " ] ";
                    ss << getSwitchOp( scriptLine->parameter(3) ) << std::endl;
                } else if ( scriptLine->parameter(0) == 2 ){
                    int tmp = RPG::variables[scriptLine->parameter(1)];
                    ss << "Operation: Set Switch[ Variable[" << scriptLine->parameter(1) << "] ] ";
                    ss << getSwitchOp( scriptLine->parameter(3) ) << std::endl << std::endl;
                    ss << "Switch[" << tmp << "] before operation = ";
                    if(RPG::switches[tmp])ss<<"ON";else ss<<"OFF";
                }
                break;
            case RPG::EVCMD_CHANGE_VARIABLE:{
                int ref1 = scriptLine->parameter(1);
                int ref2 = 0;
                int scalar = 0;
                int a, b;
                ss << "Variable Operations" << std::endl << "Operation: Variable[";
                switch( scriptLine->parameter(0) ){
                    case 0: ss << ref1 << "]"; break;
                    case 1: ss << ref1 << " - " << scriptLine->parameter(2) << "]";break;
                    case 2: ss << " Varible[" << ref1 << "] ]"; ref1 = RPG::variables[ref1]; break;
                }
                ss << varOp( scriptLine->parameter(3) );

                switch( scriptLine->parameter(4) ){
                    case 0: scalar = scriptLine->parameter(5); ss << scalar; break;
                    case 1: ref2 = scriptLine->parameter(5); ss << "Variable[" << ref2 << "]"; break;
                    case 2:
                        ref2 = scriptLine->parameter(5);
                        ss << "Variable[ Variable[ " << ref2 << "] ]";
                        ref2 = RPG::variables[ref2];
                        break;
                    case 3: ss << "Random(" << scriptLine->parameter(5) << " - " << scriptLine->parameter(6) << ")"; break;
                    case 4:
                        ss << "Item (" << RPG::getItemName( scriptLine->parameter(5) ) << "): Item ";
                        if(!scriptLine->parameter(6)) ss << "Held"; else ss << "Equipped";
                        break;
                    case 5:
                        ss << "Hero (" << RPG::actors[scriptLine->parameter(5)]->name << ") ";
                        ss << varHero( scriptLine->parameter(6) );
                        break;
                    case 6:{
                        int spr = scriptLine->parameter(5);
                        if( spr != 10005 && spr > 10000 ) ss << "Sprite (" << varSprite(spr) << "): ";
                        else if( spr == 10005 ) ss << "Event " << eventId << ": ";
                        else ss << "Event " << spr << ": ";
                        ss << varSpriteOp( scriptLine->parameter(6) );
                        break;}
                    case 7: ss << varOther( scriptLine->parameter(5) ); break;
                    default:;
                }

                ss << std::endl << std::endl;

                if( scriptLine->parameter(4) < 3){
                    a = RPG::variables[ref1];
                    if(ref2) b = RPG::variables[ref2];
                    else b = scalar;
                    ss << "Variable[" << ref1 << "]";
                    if( scriptLine->parameter(3) == 0 )
                        ss << " = " << b;
                    else{
                        ss << varOp( scriptLine->parameter(3) );
                        if(ref2) ss << "Variable[" << ref2 << "]" << std::endl;
                        else ss << scalar << std::endl;;
                        ss << a << varOp( scriptLine->parameter(3) ) << b << " = ";
                        switch( scriptLine->parameter(3) ){
                            case 0: ss << b; break;
                            case 1: ss << a + b; break;
                            case 2: ss << a - b; break;
                            case 3: ss << a * b; break;
                            case 4: ss << a / b; break;
                            case 5: ss << a % b; break;
                        }
                    }
                } else {
                    ss << "Variable[" << ref1 << "] before = " << RPG::variables[ref1];
                }
                break;
            }
            default: return true;
        }
        MessageBox(NULL,ss.str().c_str(), "Kaze EventScript Debugger" , MB_ICONINFORMATION);
    }
    return true;
}
